#version 420 core

out vec4 FragColor;

struct Light {
    vec3 position;
};  

in vec3 Color;

in vec2 texture_interpolated;

in Light transformedLights[4];

in vec3 FragPos;

uniform sampler2D texture_position;

uniform sampler2D normal_position;

uniform vec3 viewPos;

uniform float blazingFactor;

vec3 calculateLight(vec3 NormalMap, vec4 DiffuseColor, vec3 lightPos, vec3 viewDir);

void main(){

	//RGBA of our diffuse color
	vec4 DiffuseColor = texture2D(texture_position, texture_interpolated);
	
	//RGB of our normal map
	vec3 NormalMap = texture2D(normal_position, texture_interpolated).rgb;
	
	vec3 viewDir = normalize(viewPos - FragPos);

	vec3 lightSum = vec3(0,0,0);

	for(int i = 0; i < 4; i++){
	lightSum += calculateLight(NormalMap, DiffuseColor, transformedLights[i].position, viewDir);
	}

	
	FragColor = vec4(lightSum, 1.0f);

}

vec3 calculateLight(vec3 NormalMap, vec4 DiffuseColor, vec3 lightPos, vec3 viewDir){
vec2 Resolution = vec2(1024, 768);
vec4 LightColor = vec4(1.0f, 0.8f, 0.2f, 1.0f);

vec3 Falloff = vec3(0.4f, 0.3, 20.0f);

vec4 AmbientColor = vec4(0.1f, 0.1f, 0.1f, 1.0f);
	

	vec2 light = (lightPos.xy + 1.0f) / 2.0f;
	vec3 LightDir = vec3(light.xy - (gl_FragCoord.xy / Resolution.xy), 0.1);
	//Correct for aspect ratio
	LightDir.x *= Resolution.x / Resolution.y;
	
	
	//Determine distance (used for attenuation) BEFORE we normalize our LightDir
	float D = length(LightDir);
	
	//normalize our vectors
	vec3 N = normalize(NormalMap * 2.0 - 1.0);
	vec3 L = normalize(LightDir);
	
	//Pre-multiply light color with intensity
	//Then perform "N dot L" to determine our diffuse term
	vec3 Diffuse = (LightColor.rgb * LightColor.a) * max(dot(N, L), 0.0);

	//pre-multiply ambient color with intensity
	vec3 Ambient = AmbientColor.rgb * AmbientColor.a;
	
	//calculate attenuation
	float Attenuation = 1.0 / ( Falloff.x + (Falloff.y*D) + (Falloff.z*D*D) );
	
	vec3 reflectDir = reflect(-L, N);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), 64);// * 18/(2*3.141);

	//the calculation which brings it all together
	vec3 Intensity = Ambient + Diffuse * Attenuation + spec* Attenuation;
	vec3 FinalColor = DiffuseColor.rgb * Intensity * blazingFactor;
	return FinalColor;

}